/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.metadata ;

import java.util.Enumeration ;

public class Test
{
    public static void main( String args[] ) throws Exception
    {
        Test _t = new Test() ;

        _t.runTest() ;
    }

    private void runTest() throws Exception
    {
        //  test meta slot information to copy
        //  an object

        String xt0 = "Hello world" ;
        String xc0 = (String)copy( xt0 ) ;
        System.out.println(xc0) ;

        Test0 t0 = new Test0() ;
        t0.init() ;
        Test0 c0 = (Test0)copy( t0 ) ;
        c0.print() ;

        Test1 t1 = new Test1() ;
        t1.init() ;
        Test1 c1 = (Test1)copy( t1 ) ;
        c1.print() ;

        Test2 t2 = new Test2() ;
        t2.init() ;
        Test2 c2 = (Test2)copy( t2 ) ;
        c2.print() ;

        Test3 t3 = new Test3() ;
        t3.init() ;
        Test3 c3 = (Test3)copy( t3 ) ;
        c3.print() ;

        Test4 t4 = new Test4() ;
        t4.init() ;
        Test4 c4 = (Test4)copy( t4 ) ;
        c4.print() ;

        Test5 t5 = new Test5() ;
        t5.init() ;
        Test5 c5 = (Test5)copy( t5 ) ;
        c5.print() ;

    }

    Object copy( Object in ) throws Exception
    {
        MetaClass min = new MetaClass( in ) ;
        MetaClass mout = MetaClass.forName(in.getClass().getName()) ;

        System.out.println("Processing: " + in.getClass().getName()) ;

        for (Enumeration e = min.slots() ; e.hasMoreElements() ; )
        {
            MetaSlot sin = (MetaSlot)e.nextElement() ;

            MetaSlot sout = mout.getSlot( sin.getClassName(), sin.getSlotName()) ;

            System.out.println("IN: " + sin ) ;

            if (sin.isAccessible())
            {
                System.out.println("OUT: " + sout ) ;

                String signature = sin.getSignature() ;

                if (signature.charAt(0) == '[')
                {
                    switch (signature.charAt(1))
                    {
                            case 'C' :
                                sout.set(sin.charArray()) ;
                                break ;
                            case 'B' :
                                sout.set(sin.byteArray()) ;
                                break ;
                            case 'F' :
                                sout.set(sin.floatArray()) ;
                                break ;
                            case 'D' :
                                sout.set(sin.doubleArray()) ;
                                break ;
                            case 'I' :
                                sout.set(sin.intArray()) ;
                                break ;
                            case 'S' :
                                sout.set(sin.shortArray()) ;
                                break ;
                            case 'J' :
                                sout.set(sin.longArray()) ;
                                break ;
                            case 'Z' :
                                sout.set(sin.booleanArray()) ;
                                break ;
                            case 'L' :
                                sout.set(sin.getObjectArray()) ;
                                break ;
                            default :
                                throw new BadSlotType("Unsupported '" + signature + "'") ;
                        }
                    }
                    else
                    {
                        switch (signature.charAt(0))
                        {
                                case 'C' :
                                    sout.set(sin.charValue()) ;
                                    break ;
                                case 'B' :
                                    sout.set(sin.byteValue()) ;
                                    break ;
                                case 'F' :
                                    sout.set(sin.floatValue()) ;
                                    break ;
                                case 'D' :
                                    sout.set(sin.doubleValue()) ;
                                    break ;
                                case 'I' :
                                    sout.set(sin.intValue()) ;
                                    break ;
                                case 'S' :
                                    sout.set(sin.shortValue()) ;
                                    break ;
                                case 'J' :
                                    sout.set(sin.longValue()) ;
                                    break ;
                                case 'Z' :
                                    sout.set(sin.booleanValue()) ;
                                    break ;
                                case 'L' :
                                    sout.set(sin.getObject()) ;
                                    break ;
                                default :
                                    throw new BadSlotType("Unsupported '" + signature + "'") ;
                        }
                    }
                }
            }

        return mout.getObject() ;
    }
}


class Test0
{
    void init()
    {
    }

    void print()
    {
        System.out.println("\n" + this.getClass().getName() + " @" + this.hashCode() + "\n") ;
    }
}


class Test1
{
    int     a ;
    String  b ;

    void print()
    {
        System.out.println("\n" + this.getClass().getName() + " @" + this.hashCode() + "\n") ;
        System.out.println("a = " + a ) ;
        System.out.println("b = " + b ) ;
    }

    void init()
    {
        a = 1234 ;
        b = "1234" ;
    }
}

class Test2
{
    int     a ;
    int[]   b ;

    void print()
    {
        System.out.println("\n" + this.getClass().getName() + " @" + this.hashCode() + "\n") ;
        System.out.println("a = " + a ) ;
        for (int i = 0 ; i < b.length ; i++)
        {
            System.out.println("b[" + i + "] = " + b[i] ) ;
        }
    }

    void init()
    {
        a = 1234 ;

        b = new int[5] ;

        b[0] = 0 ;
        b[2] = 2 ;
        b[4] = 4 ;
    }
}

class Test3
{
    int         a ;
    Integer[]   b ;

    void print()
    {
        System.out.println("\n" + this.getClass().getName() + " @" + this.hashCode() + "\n") ;
        System.out.println("a = " + a ) ;
        for (int i = 0 ; i < b.length ; i++)
        {
            System.out.println("b[" + i + "] = " + b[i] ) ;
        }
    }

    void init()
    {
        a = 1234 ;

        b = new Integer[5] ;

        b[0] = new Integer(0) ;
        b[2] = new Integer(2) ;
        b[4] = new Integer(4) ;
    }
}


class Test4
{
    int         a ;
    Integer[]   b ;

    void print()
    {
        System.out.println("\n" + this.getClass().getName() + " @" + this.hashCode() + "\n") ;
        System.out.println("a = " + a ) ;
        if (b == null)
        {
            System.out.println("b = null") ;
        }
        else
        {
            for (int i = 0 ; i < b.length ; i++)
            {
                System.out.println("b[" + i + "] = " + b[i] ) ;
            }
        }
    }

    void init()
    {
        a = 1234 ;

        // do not initialise the array !
    }
}


class Test5
{
    int         a ;
    Integer[]   b ;

    void print()
    {
        System.out.println("\n" + this.getClass().getName() + " @" + this.hashCode() + "\n") ;
        System.out.println("a = " + a ) ;
        if (b == null)
        {
            System.out.println("b = null") ;
        }
        else
        {
            for (int i = 0 ; i < b.length ; i++)
            {
                System.out.println("b[" + i + "] = " + b[i] ) ;
            }
        }
    }

    void init()
    {
        a = 1234 ;

        b = new Integer[0] ;
    }
}


class Test6
{
    private int i0 ;
    int i1 ;
    public int i2 ;
    private static int i3 ;
    static int i4 ;
    public static int i5 ;
    transient int i6 ;

    void init()
    {
        i0 = 1 ;
        i1 = 2 ;
        i2 = 3 ;
        i3 = 4 ;
        i4 = 5 ;
        i5 = 6 ;
    }

    void print()
    {
        System.out.println("\n" + this.getClass().getName() + " @" + this.hashCode() + "\n") ;
        System.out.println("i0 (private) = " + i0 ) ;
        System.out.println("i1 (package) = " + i1 ) ;
        System.out.println("i2 (public) = " + i2 ) ;
        System.out.println("i3 (private static) = " + i3 ) ;
        System.out.println("i4 (package static) = " + i4 ) ;
        System.out.println("i5 (public static) = " + i5 ) ;
        System.out.println("i6 (transient) = " + i6 ) ;
    }
}

