/* MetaClass.c */

#include    "JOP_metadata_MetaClass.h"
#include    "JOP_metadata_MetaSlot.h"

/**
    This method creates an instance of an object of the
    desired class. WARNING: This method does not
    check security and does not execute the classes
    constructor.
*/

extern struct Hjava_lang_Object *JOP_metadata_MetaClass_newInstance(struct HJOP_metadata_MetaClass *_this,struct Hjava_lang_Class *aClass)
{
    ExecEnv     *ee = EE() ;
    ClassClass  *cd ;

    if (aClass == NULL)
    {
        return NULL ;
    }

    cd = unhand(aClass) ;

    if (cd == NULL)
    {
        return NULL ;
    }

    /* now allocate the object and return it */

    return newobject( cd, 0, ee ) ;
}

/**
    Compare two classnames ignoring the difference between '.' and '/'
*/

int compareClassname( char *n1, char *n2 )
{
    while (*n1 && *n2)
    {
        if (*n1 == '.' || *n1 == '/')
        {
            if (*n2 != '.' && *n2 != '/')
                return 1 ;
        }
        else if (*n1 != *n2)
        {
            return 1 ;
        }

        n1++ ; n2++ ;
    }

    if (*n1 == *n2)
        return 0 ;
    else
        return 1 ;
}


ClassClass *getClass( struct Hjava_lang_Object *o, char *cname )
{
    //  given a class definition, walk up the definition until the
    //  classname matches the passed classname

    ClassClass  *cd ;

    // printf("getClass( %s )\n", cname ) ;

    cd = o->methods->classdescriptor ;

    while (cd != NULL)
    {
        // printf("Checking %s against %s\n", cd->name, cname ) ;

        if (compareClassname( cname, cd->name ) == 0)
        {
            return cd ;
        }

        // printf("Super name = %s\n", cd->super_name) ;

        if (cd->superclass == NULL)
        {
            cd = NULL ;
        }
        else
        {
            cd = cd->superclass->obj ;
        }
    }

    // printf("ClassClass %s not found\n", cname) ;

    return NULL ;
}

extern int32_t JOP_metadata_MetaClass_getSlotCount(struct HJOP_metadata_MetaClass *_this,struct Hjava_lang_String *classname)
{
    ClassClass              *cd ;

    // printf("GET COUNT %08lx %08lx ( %s )\n", _this, classname, makeCString(classname)) ;

    cd = getClass( _this->obj->obj, makeCString(classname) ) ;

    if (cd == NULL)
    {
        return 0 ;
    }

    return cd->fields_count ;
}

extern struct Hjava_lang_String *JOP_metadata_MetaClass_getSlotSignature(struct HJOP_metadata_MetaClass *_this,struct Hjava_lang_String *classname,int32_t slot)
{
    ClassClass              *cd ;
    struct fieldblock       *f ;

    // printf("GET SIGNATURE %08lx %08lx %d\n", _this, classname, slot ) ;

    cd = getClass( _this->obj->obj, makeCString(classname) ) ;

    if (cd == NULL)
    {
        return 0 ;
    }

    f = cd->fields ;

    return makeJavaString(fieldsig(f + slot), strlen(fieldsig(f + slot)));
}

extern struct Hjava_lang_String *JOP_metadata_MetaClass_getSlotName(struct HJOP_metadata_MetaClass *_this,struct Hjava_lang_String *classname,int32_t slot)
{
    ClassClass          *cd ;
    struct fieldblock   *f ;

    // printf("GET NAME %08lx %08lx %d\n", _this, classname, slot ) ;

    cd = getClass( _this->obj->obj, makeCString(classname) ) ;

    if (cd == NULL)
    {
        return 0 ;
    }

    f = cd->fields ;

    return makeJavaString(fieldname(f + slot), strlen(fieldname(f + slot)));
}

extern int32_t JOP_metadata_MetaClass_getSlotAccess(struct HJOP_metadata_MetaClass *_this,struct Hjava_lang_String *classname,int32_t slot)
{
    ClassClass          *cd ;
    struct fieldblock   *f ;

    // printf("GET ACCESS %08lx %08lx %d\n", _this, classname, slot ) ;

    cd = getClass( _this->obj->obj, makeCString(classname) ) ;

    if (cd == NULL)
    {
        return 0 ;
    }

    f = cd->fields ;

    return (f+slot)->access ;
}

/**************************** SLOT VALUE ACCESS ****************************/

extern /*boolean*/ int32_t JOP_metadata_MetaSlot_getBooleanValue(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    int32_t                *fptr ;

    // printf("GET Z %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (int32_t *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern int64_t JOP_metadata_MetaSlot_getLongValue(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    int64_t             *fptr ;

    // printf("GET J %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (int64_t *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern short JOP_metadata_MetaSlot_getShortValue(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    short               *fptr ;

    // printf("GET S %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (short *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern int32_t JOP_metadata_MetaSlot_getIntValue(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    int32_t                *fptr ;

    // printf("GET I %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (int32_t *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern float JOP_metadata_MetaSlot_getFloatValue(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    float               *fptr ;

    // printf("GET F %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (float *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern double JOP_metadata_MetaSlot_getDoubleValue(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    double              *fptr ;

    // printf("GET D %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (double *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern char JOP_metadata_MetaSlot_getByteValue(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    char                *fptr ;

    // printf("GET B %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (char *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern unicode JOP_metadata_MetaSlot_getCharValue(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    unicode             *fptr ;

    // printf("GET C %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (unicode *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern struct Hjava_lang_Object *JOP_metadata_MetaSlot_getObjectValue(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    Hjava_lang_Object   **fptr ;

    // printf("GET L %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (Hjava_lang_Object **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}


extern /*boolean*/ HArrayOfInt *JOP_metadata_MetaSlot_getBooleanArray(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfInt         **fptr ;

    // printf("GET [Z %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfInt **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern HArrayOfLong *JOP_metadata_MetaSlot_getLongArray(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfLong        **fptr ;

    // printf("GET [J %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfLong **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern HArrayOfShort *JOP_metadata_MetaSlot_getShortArray(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfShort       **fptr ;

    // printf("GET [S %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfShort **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern HArrayOfInt *JOP_metadata_MetaSlot_getIntArray(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfInt         **fptr ;

    // printf("GET [I %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfInt **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern HArrayOfFloat *JOP_metadata_MetaSlot_getFloatArray(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfFloat       **fptr ;

    // printf("GET [F %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfFloat **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern HArrayOfDouble *JOP_metadata_MetaSlot_getDoubleArray(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfDouble      **fptr ;

    // printf("GET [D %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfDouble **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern HArrayOfByte *JOP_metadata_MetaSlot_getByteArray(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfByte        **fptr ;

    // printf("GET [B %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfByte **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern HArrayOfChar *JOP_metadata_MetaSlot_getCharArray(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfChar        **fptr ;

    // printf("GET [C %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfChar **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}

extern HArrayOfObject *JOP_metadata_MetaSlot_getObjectArrayNative(struct HJOP_metadata_MetaSlot *_this)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfObject      **fptr ;

    // printf("GET [L %08lx %08lx %08lx\n", _this, _this->obj, _this->obj->classname ) ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfObject **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    return *fptr ;
}


/***************** S E T ****************/

extern void JOP_metadata_MetaSlot_setCharValue(struct HJOP_metadata_MetaSlot *_this,unicode v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    unicode             *fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (unicode *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}

extern void JOP_metadata_MetaSlot_setBooleanValue(struct HJOP_metadata_MetaSlot *_this,/*boolean*/ int32_t v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    int32_t                *fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (int32_t *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}

extern void JOP_metadata_MetaSlot_setLongValue(struct HJOP_metadata_MetaSlot *_this,int64_t v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    int64_t             *fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (int64_t *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}

extern void JOP_metadata_MetaSlot_setShortValue(struct HJOP_metadata_MetaSlot *_this, short v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    int32_t                *fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (int32_t *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}

extern void JOP_metadata_MetaSlot_setIntValue(struct HJOP_metadata_MetaSlot *_this,int32_t v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    int32_t                *fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (int32_t *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}

extern void JOP_metadata_MetaSlot_setFloatValue(struct HJOP_metadata_MetaSlot *_this,float v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    float               *fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (float *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}

extern void JOP_metadata_MetaSlot_setDoubleValue(struct HJOP_metadata_MetaSlot *_this,double v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    double              *fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (double *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}

extern void JOP_metadata_MetaSlot_setByteValue(struct HJOP_metadata_MetaSlot *_this, char v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    int32_t                *fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (int32_t *)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}

extern void JOP_metadata_MetaSlot_setObjectValue(struct HJOP_metadata_MetaSlot *_this, struct Hjava_lang_Object *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    Hjava_lang_Object   **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (Hjava_lang_Object **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}


extern void JOP_metadata_MetaSlot_setCharArray(struct HJOP_metadata_MetaSlot *_this,HArrayOfChar *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfChar        **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfChar **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}


extern void JOP_metadata_MetaSlot_setBooleanArray(struct HJOP_metadata_MetaSlot *_this,HArrayOfInt *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfInt         **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfInt **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}


extern void JOP_metadata_MetaSlot_setLongArray(struct HJOP_metadata_MetaSlot *_this,HArrayOfLong *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfLong        **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfLong **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}


extern void JOP_metadata_MetaSlot_setShortArray(struct HJOP_metadata_MetaSlot *_this,HArrayOfShort *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfShort       **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfShort **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}


extern void JOP_metadata_MetaSlot_setIntArray(struct HJOP_metadata_MetaSlot *_this,HArrayOfInt *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfInt         **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfInt **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}


extern void JOP_metadata_MetaSlot_setFloatArray(struct HJOP_metadata_MetaSlot *_this,HArrayOfFloat *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfFloat       **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfFloat **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}


extern void JOP_metadata_MetaSlot_setDoubleArray(struct HJOP_metadata_MetaSlot *_this,HArrayOfDouble *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfDouble      **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfDouble **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}

extern void JOP_metadata_MetaSlot_setByteArray(struct HJOP_metadata_MetaSlot *_this,HArrayOfByte *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfByte        **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfByte **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}


extern void JOP_metadata_MetaSlot_setObjectArray(struct HJOP_metadata_MetaSlot *_this,HArrayOfObject *v)
{
    ClassClass          *cd ;
    struct fieldblock   *fblk ;
    HArrayOfObject      **fptr ;

    cd = getClass( _this->obj->object, makeCString(_this->obj->classname) ) ;

    fblk = cd->fields ;

    fptr = (HArrayOfObject **)((char *)_this->obj->object->obj + (fblk + _this->obj->index)->u.offset) ;

    *fptr = v ;
}


