/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist ;

import  JOP.persist.ObjectIdentifier ;

/**
    A definition manager is used to return an object's
    definition from either an object instance or
    from the name of the class of the object. The definition
    manager interface is called from PersistManager.
    There are currently three types of definition managers:
    <p>
    1)  Persistable
    <p>
    2)  PersistMarshall
    <p>
    3)  Native
*/

public class DefinitionManager
{
    /**
        Get an object definition structure based on
        the passed object.
    */
    static ObjectDefinition getObjectDefinition( Object o ) throws ClassNotFoundException, Exception
    {
        ObjectDefinition od = getDefinition( o.getClass() ) ;

        od.setObject( o ) ;

        return od ;
    }

    /**
        Get an empty object definition. The object will be
        created by the definition controller. This can
        happen immediately or at a later time.
    */
    static ObjectDefinition getObjectDefinition( String signature ) throws Exception
    {
        ObjectDefinition od = getDefinition( Class.forName( signature )) ;

        od.createObject() ;

        return od ;
    }

    private static ObjectDefinition getDefinition( Class c ) throws ClassNotFoundException, Exception
    {
        Class persistable = Class.forName("JOP.persist.Persistable") ;

        Class temp = c ;

        while (temp != null)
        {
            Class interfaces[] = temp.getInterfaces() ;

            for (int i = 0 ; i < interfaces.length ; i++)
            {
                if (persistable.equals(interfaces[i]))
                {
                    return new PersistableObjectDefinition( c ) ;
                }
            }

            // check to see if we have inherited Persistable

            temp = temp.getSuperclass() ;
        }

        try
        {
            String marshaller = "JOP.persistmarshall." + c.getName() + "_JOP" ;

            Class pmarshall = Class.forName(marshaller) ;

            return new MarshallObjectDefinition(pmarshall, c ) ;
        }
        catch (ClassNotFoundException ex)
        {
            //  we must check that none of the superclasses can
            //  be custom marshalled otherwise strange results
            //  will occur

            Class test = c.getSuperclass() ;

            while (test != null)
            {
                try
                {
                    String tempname = "JOP.persistmarshall." + test.getName() + "_JOP" ;

                    Class.forName(tempname) ;

                    throw new PersistException("A superclass of '" + c.getName() + "' has a customer marshaller") ;

                } catch (ClassNotFoundException fun)
                {

                }

                test = test.getSuperclass() ;
            }
            return new NativeObjectDefinition(c) ;
        }
    }
}

