/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist ;

/**
    This class provides static methods to return whether a
    signature is one of the special 'inline' Java types and
    to return the constant that represents the type.
*/

public class JavaTypes
{
    /*
        Constants for referring to the types
    */
    public static final int TYPE_UNKNOWN = 0 ;
    public static final int TYPE_OBJECT = 1 ;

    public static final int TYPE_SHORT = 2 ;
    public static final int TYPE_LONG = 3 ;
    public static final int TYPE_INTEGER = 4 ;
    public static final int TYPE_BYTE = 5 ;
    public static final int TYPE_CHARACTER = 6 ;
    public static final int TYPE_BOOLEAN = 7 ;
    public static final int TYPE_FLOAT = 8 ;
    public static final int TYPE_DOUBLE = 9 ;
    public static final int TYPE_STRING = 10 ;
    public static final int TYPE_DATE = 11 ;

    /**
        English names of the types

        Index of array is the TYPE_xxxx constant.
    */
    public static final String TypeName[] =
        {
            "n/a",                  // TYPE_UNKNOWN
            "Object",               // TYPE_OBJECT
            "Short",                // TYPE_SHORT
            "Long",                 // TYPE_LONG
            "Integer",              // TYPE_INTEGER
            "Byte",                 // TYPE_BYTE
            "Character",            // TYPE_CHARACTER
            "Boolean",              // TYPE_BOOLEAN
            "Float",                // TYPE_FLOAT
            "Double",               // TYPE_DOUBLE
            "String",               // TYPE_STRING
            "Date"                  // TYPE_DATE
        } ;

    /*
        Equivalent signature for each type.

        Index of array is the TYPE_xxxx constant
    */

    public static final String TypeSignature[] =
        {
            "Ljava.lang.Object",        // TYPE_UNKNOWN
            "Ljava.lang.Object",        // TYPE_OBJECT
            "Ljava.lang.Integer",       // TYPE_SHORT
            "Ljava.lang.Long",          // TYPE_LONG
            "Ljava.lang.Integer",       // TYPE_INTEGER
            "Ljava.lang.Integer",       // TYPE_BYTE
            "Ljava.lang.Character",     // TYPE_CHARACTER
            "Ljava.lang.Boolean",       // TYPE_BOOLEAN
            "Ljava.lang.Float",         // TYPE_FLOAT
            "Ljava.lang.Double",        // TYPE_DOUBLE
            "Ljava.lang.String",        // TYPE_STRING
            "Ljava.util.Date",          // TYPE_DATE
        } ;

    /*
        Java class names for the special basic types
    */
    public static final String ClassSignature[] =
        {
            "Ljava.lang.Integer",
            "Ljava.lang.Long",
            "Ljava.lang.Character",
            "Ljava.lang.Boolean",
            "Ljava.lang.Float",
            "Ljava.lang.Double",
            "Ljava.lang.String",
            "Ljava.util.Date"
        } ;

    public static final int ClassSignatureType[] =
        {
            TYPE_INTEGER,
            TYPE_LONG,
            TYPE_CHARACTER,
            TYPE_BOOLEAN,
            TYPE_FLOAT,
            TYPE_DOUBLE,
            TYPE_STRING,
            TYPE_DATE
        } ;

    /*
        Signatures for the built-in scalar types
    */
    public static final String ScalarSignature[] =
        {
            "S",
            "J",
            "I",
            "B",
            "C",
            "Z",
            "F",
            "D",
        } ;

    public static final int ScalarSignatureType[] =
        {
            TYPE_SHORT,
            TYPE_LONG,
            TYPE_INTEGER,
            TYPE_BYTE,
            TYPE_CHARACTER,
            TYPE_BOOLEAN,
            TYPE_FLOAT,
            TYPE_DOUBLE
        } ;

    /**
        Return the in-built type of the given signature or
        return TYPE_UNKNOWN if the signature is not known.
    */

    public static int getSignatureType( String classname )
    {
        if (classname.length() == 1)
        {
            for (int i = 0 ; i < ScalarSignature.length ; i++)
            {
                if (ScalarSignature[i].equals(classname))
                {
                    return ScalarSignatureType[i] ;
                }
            }
        }

        int type = TYPE_OBJECT ;

        if (classname.charAt(0) != 'L')
        {
            classname = "L" + classname.replace('/','.') ;
        }
        else
        {
            classname = classname.replace('/','.') ;
        }

        for (int i = 0 ; i < ClassSignature.length ; i++)
        {
            if (ClassSignature[i].equals(classname))
            {
                return ClassSignatureType[i] ;
            }
        }

        return type ;
    }
}
