/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist ;

import  java.util.Enumeration ;
import  java.util.Vector ;
import  java.util.Hashtable ;

public class MarshallObjectDefinition extends PersistableObjectDefinition implements ObjectDefinition
{
    private PersistMarshall     marshaller ;
    private Class               targetClass ;

    MarshallObjectDefinition( Class marshallClass, Class c ) throws Exception
    {
        super(c) ;

        // System.out.println("CUSTOM MARSHALL FOR " + c.getName()) ;

        marshaller = (PersistMarshall)marshallClass.newInstance() ;
        targetClass = c ;
    }

    /**
        Called by the persist manager after the object has
        been read from the persist storage and after all
        calls to 'aSlot.setValue()' have been completed.
    */
    public void endInput() throws Exception
    {
        if (marshaller.canModify())
        {
            PodInCntl cntl = new PodInCntl( this ) ;

            marshaller.streamIn( cntl ) ;
        }
    }

    /**
        Called by the persist manager before the object
        is written to the persist storage and before any
        call to 'aSlot.getValue()'
    */
    public void beginOutput() throws Exception
    {
        //  reads the object by calling target.streamOut()
        //  into a temporary array of values, each value
        //  has references to the slot definition.

        PodOutCntl cntl = new PodOutCntl( this ) ;

        marshaller.streamOut( cntl ) ;
    }

    /**
        Set the object for which this definition manager
        must provide access. This will occur if we are
        reading the object or updating an existing
        object.
    */
    public void setObject(Object obj) throws Exception
    {
        marshaller.setObject( obj ) ;
    }

    /**
        Returns the object created by this controller
    */
    public Object getObject() throws Exception
    {
        //  call target.streamIn( controller ) using the
        //  array of slot values and then resets the
        //  array of slot values.

        if (!marshaller.canModify())
        {
            PodInCntl cntl = new PodInCntl( this ) ;

            marshaller.streamIn( cntl ) ;
        }

        return marshaller.getObject() ;
    }

    /**
        Gets the name of the class of the object as a String.
        The classname is as returned by Class.getName().
    */
    public String getClassName()
    {
        return targetClass.getName() ;
    }

    /**
        Here is where we stream in the definitions of the
        classes
    */
    public Enumeration classes()
    {
        if (classes == null)
        {
            classes = new Hashtable() ;

            try
            {
                Hashtable cdefs = PodDefnCntl.getDefinition(marshaller) ;

                for (Enumeration e = cdefs.elements() ; e.hasMoreElements() ; )
                {
                    PodDefnCntlClass cdef = (PodDefnCntlClass)e.nextElement() ;

                    classes.put(cdef.classname, new PodClassDefinition(cdef)) ;
                }
            }
            catch (Exception ex)
            {
                return classes.elements() ;
            }

        }
        return classes.elements() ;
    }
}

