/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist ;

import java.util.Enumeration ;

/**
    This interface is used by the media specific
    persist managers to access the definition and
    slots of an object.
*/

public interface ObjectDefinition
{
    /**
        Allow the marshalling mechanism to invoke the
        PersistManager if necessary.
     */
     public void setPersistManager( PersistManager p ) ;

    /**
        Called by the persist manager before the object
        is read from the persist storage and before any
        call to 'aSlot.setValue()'
    */
    public void beginInput() throws Exception ;

    /**
        Called by the persist manager after the object has
        been read from the persist storage and after all
        calls to 'aSlot.setValue()' have been completed.
    */
    public void endInput() throws Exception ;

    /**
        Called by the persist manager before the object
        is written to the persist storage and before any
        call to 'aSlot.getValue()'
    */
    public void beginOutput() throws Exception ;

    /**
        Called by the persist manager after the object has
        been written to the persist storage and after all
        calls to 'aSlot.getValue()' have been completed.
    */
    public void endOutput() throws Exception ;

    /**
        Set the object for which this definition manager
        must provide access.
    */
    public void setObject(Object obj) throws Exception ;

    /**
        Tell the definition manager to create an instance
        of the object.
    */
    public void createObject() throws Exception ;

    /**
        Returns the object created by this controller
    */
    public Object getObject() throws Exception ;

    /**
        Return a checksum of the slots of the object. This
        value is used by the PersistManager to determine
        whether the object has changed. Do not return a
        checksum value of 0.
    */
    public long getChecksum() throws Exception ;

    /**
        Gets the name of the class of the object as a String.
        The classname is as returned by Class.getName().
    */
    public String getClassName() ;

    /**
        Gets the class definitions for this object. The
        class defintiions include the current class
        and all of the super classes. Classnames must be
        returned in order from the current class, the
        current classes superclass, up to java.lang.Object.
    */
    public Enumeration classes() ;
}

