/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist ;

import  java.net.InetAddress ;
import  java.util.Random ;

/**
    Assigns a unique Object Identifier. The current
    algorithm uses the current time, a random
    sequence number and the hashcode of the IP address.
    <p>
    A band of reserved object identifiers is kept for
    pre-defined use.
*/

class OidGenerator
{
    /**
        Reserved OID's that should not be generated.
    */
    static final long NULL = 0 ;

    /**
        Low limit of OID's suitable for use by
        user applications.
    */
    static final long USER_LOW = 32768 ;

    /**
        High limit of OID's suitable for use by
        user applications.
    */
    static final long USER_HIGH = 65535 ;

    /**
        The first 65536 oids are reserved
    */
    private static final long   RESERVED = 65536 ;

    static final long   JOP_ROOT = USER_LOW - 1 ;

    private static long _seq = 0 ;
    private static boolean initialised = false ;
    private static long _iphash = 0 ;

    /**
        Assign a new unique object identifier. Identifiers in
        the reserved band will not be assigned.
    */
    synchronized public static long newOid()
    {
        long    oid = RESERVED - 1 ;

        while (oid < RESERVED)
        {
            if (!initialised)
            {
                _seq = new Random().nextLong() << 48 ;

                try
                {
                    long _iphash = InetAddress.getLocalHost().hashCode() ;

                    _iphash = _iphash << 32 ;
                } catch (Exception ex)
                {
                    _iphash = 0 ;
                }

                initialised = true ;
            }

            oid = _iphash + _seq++ + System.currentTimeMillis() ;

            if (oid < 0)
            {
                oid = oid * -1 ;
            }
        }

        return oid ;
    }
}
