/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist ;

/**
    A class implementation that supports this interface
    can act as a custom marshaller for an object. This
    is useful where the source to the object is not
    available, where you don't want to modify the source or
    where native methods are not suitable.
    <p>
    The PersistMarshall interface also allows an object
    that naturally consists of an array of instances to
    be marshalled and stored efficiently. Examples of this
    include Hashtables, Vectors and Arrays.
    <p>
    A custom marshaller class must:
    <p>
    <ol>
    <li>Implement this interface
    <li>Have an empty, public constructor
    <li>Have a name the same as the class that it is marshalling
    with thr suffix '_JOP'.
    <li>Must appear in a package that is the same relationship to
    JOP.persistmarshall as the original object is to CLASSPATH (eg the custom marshall
    class for 'java.lang.String' is 'JOP.persistmarshall.java.lang.String_JOP')
    </ol>
*/
public interface PersistMarshall extends Persistable
{
    /**
        Called by the PersistManager to initialise the
        customer marshaller when reading an object where the
        object already exists.
        <p>The customer marshaller must reset and overwrite the
        slot values of the existing object.
        <p>
        *@param o   an existing object
    */
    public void setObject(Object o) throws Exception ;

    /**
        Invoked by the PersistManager to retrieve an object
        that was created by the customer marshaller. Called
        after initInput(). Can also be called after initInput(o)
        in which case the original object should be returned.
    */
    public Object getObject() throws Exception ;

    /**
        Returns TRUE if the object can be modified by the custom
        marshaller after it has been created and returned by
        'getObject()'. This method is used to choose the order in
        which the 'streamIn()' method is invoked, ie before or after
        'getObject()'.
        <p>
        If the object to be marhshalled can't be modified once created
        AND the object references other objects then errors will occur. In
        this case you should not use the custom marshall, but rather use
        either Persistable or a native marshall.
    */
    public boolean canModify() ;
}

