/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist ;

import  java.util.Vector ;
import  java.util.Enumeration ;
import  java.util.Date ;

/**
    This is an internal class used by JOP to support arrays when using the
    Persistable interface.
*/

class ArrayElement
{
    Object              obj ;
    ObjectIdentifier    oid ;

    ArrayElement()
    {
    }

    ArrayElement( Object o )
    {
        obj = o ;
    }
}

class PersistableArray
{
    PersistSlotType     type ;
    Vector              v ;

    PersistableArray( PersistSlotType t )
    {
        type = t ;
        v = new Vector() ;
    }

    public String toString()
    {
        return "(" + type + ")" + v.toString() ;
    }

    int size()
    {
        return v.size() ;
    }

    void setRefOid( int n, ObjectIdentifier oid )
    {
        while (v.size() <= n)
        {
            ArrayElement element = new ArrayElement() ;
            v.insertElementAt( element, v.size()) ;
        }

        ArrayElement m = (ArrayElement)v.elementAt(n) ;

        // System.out.println("Array ref[" + n + "] set to " + oid ) ;

        m.oid = oid ;
    }

    ObjectIdentifier getRefOid( int n )
    {
        ArrayElement    m = (ArrayElement)v.elementAt(n) ;

        if (m == null)
        {
            return null ;
        }
        else
        {
            return m.oid ;
        }
    }

    void setValue( int n, Object value )
    {
        while (v.size() <= n)
        {
            ArrayElement element = new ArrayElement() ;
            v.insertElementAt( element, v.size()) ;
        }

        ArrayElement m = (ArrayElement)v.elementAt(n) ;

        // System.out.println("Array obj[" + n + "] set to " + value ) ;

        m.obj = value ;
    }

    Object getValue( int n )
    {
        ArrayElement    m = (ArrayElement)v.elementAt(n) ;

        if (m == null)
        {
            return null ;
        }
        else
        {
            return m.obj ;
        }
    }


    int[] intValueArray() throws Exception
    {
        if (type != PersistController.TYPE_INTEGER)
        {
            throw new PersistException("Bad array element type") ;
        }

        int value[] ;

        value = new int[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = ((Integer)m.obj).intValue() ;
        }

        return value ;
    }

    char[] charValueArray() throws Exception
    {
        if (type != PersistController.TYPE_CHARACTER)
        {
            throw new PersistException("Bad array element type") ;
        }

        char value[] ;

        value = new char[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = ((Character)m.obj).charValue() ;
        }

        return value ;
    }

    byte[] byteValueArray() throws Exception
    {
        if (type != PersistController.TYPE_BYTE)
        {
            throw new PersistException("Bad array element type") ;
        }

        byte value[] ;

        value = new byte[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (byte)((Integer)m.obj).intValue() ;
        }

        return value ;
    }

    short[] shortValueArray() throws Exception
    {
        if (type != PersistController.TYPE_SHORT)
        {
            throw new PersistException("Bad array element type") ;
        }

        short value[] ;

        value = new short[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (short)((Integer)m.obj).intValue() ;
        }

        return value ;
    }

    long[] longValueArray() throws Exception
    {
        if (type != PersistController.TYPE_LONG)
        {
            throw new PersistException("Bad array element type") ;
        }

        long value[] ;

        value = new long[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = ((Long)m.obj).longValue() ;
        }

        return value ;
    }

    float[] floatValueArray() throws Exception
    {
        if (type != PersistController.TYPE_FLOAT)
        {
            throw new PersistException("Bad array element type") ;
        }

        float value[] ;

        value = new float[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = ((Float)m.obj).floatValue() ;
        }

        return value ;
    }

    double[] doubleValueArray() throws Exception
    {
        if (type != PersistController.TYPE_DOUBLE)
        {
            throw new PersistException("Bad array element type") ;
        }

        double value[] ;

        value = new double[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = ((Double)m.obj).doubleValue() ;
        }

        return value ;
    }

    boolean[] booleanValueArray() throws Exception
    {
        if (type != PersistController.TYPE_BOOLEAN)
        {
            throw new PersistException("Bad array element type") ;
        }

        boolean value[] ;

        value = new boolean[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++ )
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = ((Boolean)m.obj).booleanValue() ;
        }

        return value ;
    }

    Date[] getDateArray() throws Exception
    {
        if (type != PersistController.TYPE_DATE)
        {
            throw new PersistException("Bad array element type") ;
        }

        Date value[] ;

        value = new Date[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++ )
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (Date)m.obj ;
        }

        return value ;
    }

    Character[] getCharacterArray() throws Exception
    {
        if (type != PersistController.TYPE_CHARACTER)
        {
            throw new PersistException("Bad array element type") ;
        }

        Character value[] ;

        value = new Character[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (Character)m.obj ;
        }

        return value ;
    }

    String[] getStringArray() throws Exception
    {
        if (type != PersistController.TYPE_STRING)
        {
            throw new PersistException("Bad array element type") ;
        }

        String value[] ;

        value = new String[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (String)m.obj ;
        }

        return value ;
    }

    Integer[] getIntegerArray() throws Exception
    {
        if (type != PersistController.TYPE_INTEGER)
        {
            throw new PersistException("Bad array element type") ;
        }

        Integer value[] ;

        value = new Integer[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++)
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (Integer)m.obj ;
        }

        return value ;
    }

    Long[] getLongArray() throws Exception
    {
        if (type != PersistController.TYPE_LONG)
        {
            throw new PersistException("Bad array element type") ;
        }

        Long value[] ;

        value = new Long[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++ )
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (Long)m.obj ;
        }

        return value ;
    }

    Float[] getFloatArray() throws Exception
    {
        if (type != PersistController.TYPE_FLOAT)
        {
            throw new PersistException("Bad array element type") ;
        }

        Float value[] ;

        value = new Float[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++ )
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (Float)m.obj ;
        }

        return value ;
    }

    Double[] getDoubleArray() throws Exception
    {
        if (type != PersistController.TYPE_DOUBLE)
        {
            throw new PersistException("Bad array element type") ;
        }

        Double value[] ;

        value = new Double[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++ )
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (Double)m.obj ;
        }

        return value ;
    }

    Boolean[] getBooleanArray() throws Exception
    {
        if (type != PersistController.TYPE_BOOLEAN)
        {
            throw new PersistException("Bad array element type") ;
        }

        Boolean value[] ;

        value = new Boolean[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++ )
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (Boolean)m.obj ;
        }

        return value ;
    }

    Object[] getObjectArray() throws Exception
    {
        if (type != PersistController.TYPE_OBJECT)
        {
            throw new PersistException("Bad array element type") ;
        }

        Object value[] ;

        value = new Object[v.size()] ;

        int i = 0 ;

        for (Enumeration e = v.elements() ; e.hasMoreElements() ; i++ )
        {
            ArrayElement    m = (ArrayElement)e.nextElement() ;

            value[i] = (Object)m.obj ;
        }

        return value ;
    }

    void putArray(int value[]) throws Exception
    {
        if (type != PersistController.TYPE_INTEGER)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( new Integer(value[i]))) ;
            }
        }
    }

    void putArray(char value[]) throws Exception
    {
        if (type != PersistController.TYPE_CHARACTER)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( new Character(value[i]))) ;
            }
        }
    }

    void putArray(byte value[]) throws Exception
    {
        if (type != PersistController.TYPE_BYTE)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( new Integer(value[i]))) ;
            }
        }
    }

    void putArray(short value[]) throws Exception
    {
        if (type != PersistController.TYPE_SHORT)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( new Integer(value[i]))) ;
            }
        }
    }

    void putArray(long value[]) throws Exception
    {
        if (type != PersistController.TYPE_LONG)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( new Long(value[i]))) ;
            }
        }
    }

    void putArray(float value[]) throws Exception
    {
        if (type != PersistController.TYPE_FLOAT)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( new Float(value[i]))) ;
            }
        }
    }

    void putArray(double value[]) throws Exception
    {
        if (type != PersistController.TYPE_DOUBLE)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( new Double(value[i]))) ;
            }
        }
    }

    void putArray(boolean value[]) throws Exception
    {
        if (type != PersistController.TYPE_BOOLEAN)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( new Boolean(value[i]))) ;
            }
        }
    }

    void putArray(String value[]) throws Exception
    {
        if (type != PersistController.TYPE_STRING)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( value[i] )) ;
            }
        }
    }

    void putArray(Date value[]) throws Exception
    {
        if (type != PersistController.TYPE_DATE)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( value[i] )) ;
            }
        }
    }

    void putArray(Integer value[]) throws Exception
    {
        if (type != PersistController.TYPE_INTEGER)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( value[i] )) ;
            }
        }
    }

    void putArray(Character value[]) throws Exception
    {
        if (type != PersistController.TYPE_CHARACTER)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( value[i] )) ;
            }
        }
    }

    void putArray(Long value[]) throws Exception
    {
        if (type != PersistController.TYPE_LONG)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( value[i] )) ;
            }
        }
    }

    void putArray(Float value[]) throws Exception
    {
        if (type != PersistController.TYPE_FLOAT)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( value[i] )) ;
            }
        }
    }

    void putArray(Double value[]) throws Exception
    {
        if (type != PersistController.TYPE_DOUBLE)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( value[i] )) ;
            }
        }
    }

    void putArray(Boolean value[]) throws Exception
    {
        if (type != PersistController.TYPE_BOOLEAN)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( value[i] )) ;
            }
        }
    }

    void putArray(Object value[]) throws Exception
    {
        if (type != PersistController.TYPE_OBJECT)
        {
            throw new PersistException("Bad array element type") ;
        }

        if (value != null)
        {
            for (int i = 0 ; i < value.length ; i++)
            {
                v.addElement( new ArrayElement( value[i] )) ;
            }
        }
    }
}
