/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist ;

/**
    For each slot of the object, the SlotDefinition
    allows the slots type and name to be determined
    and allows the current value of the slot to
    be stored or retrieved.
*/

public interface SlotDefinition
{
    /**
        Get the Slot's signature (Java signature as per
        signature.h)
    */
    public String getSignature();

    /**
        Get the Slot's signature as an integer type code
        (as per JOP.persist.JavaTypes).
    */
    public int getSignatureType();

    /**
        Get the name of this slot
    */
    public String getName() ;

    /**
        Return an object that stores the slots value. Note
        that where the original type of the slot was an
        object reference the Object returned will be
        of type ObjectIdentifier - and not the object.
    */
    public Object getValue() throws Exception ;

    /**
        Allows the slot's value to be set. If the slot
        is originally a reference to a mutable object
        then the Object must be of type ObjectIdentifier.
        <p>
        Can only called if the slot is not an array
    */
    public void setValue( Object obj ) throws Exception ;

    /**
        For object references then an object id will be
        assigned to the referenced object. The object id
        will be null for all immutable objects and for
        scalars.
        <p>
        Can only be called if the slot is not an array
    */
    public void setRefOid( ObjectIdentifier oid );

    /**
        Test if the slot is an array
    */

    public boolean isArray();

    /**
        Return the length of the array.
        <p>
        Can only called if the slot is an array.
    */
    public int sizeArray();

    /**
        Return 'true' if the array is null
    */
    public boolean isNullArray() ;

    /**
        Set the array to be a null array
    */
    public void setNullArray() ;

    /**
        Fetch the 'nth' element. Can only be called
        if the slot is an array
    */

    public Object getValue( int n );

    /**
        Set the 'nth' element of an array.
    */

    public void setValue( int n, Object obj ) ;

    /**
        For object references then an object id will be
        assigned to the referenced object. The object id
        will be null for all immutable objects and for
        scalars.
        <p>
        Can only be called if the slot is not an array
    */
    public void setRefOid( int n, ObjectIdentifier oid );

    /**
        Return the assigned RefOid for this object
    */
    public ObjectIdentifier getRefOid() ;

    /**
        Return the assigned RefOid for each slot in an array
    */
    public ObjectIdentifier getRefOid(int n);
}

