/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist.jdbc ;

import java.net.URL;
import java.sql.*;

import  JOP.persist.JavaTypes ;

class JdbcIdTable
{
    JdbcPersistManager   parent ;

    public JdbcIdTable(JdbcPersistManager p) throws Exception
    {
        parent = p ;

        defineIdTable() ;
    }

    /******************** OID table access ******************/

    /**
        Try to read the existing object id table. This table
        contains object id's and the type of the object (stored
        as a fully qualified Java class name). If the table
        already exists then the next object id value is
        saved, otherwise if the table does not exist the
        table is created.
    */
    private void defineIdTable() throws Exception
    {
        /*
            Check to see if the id table exists.
        */

        Statement dbstmt1 = null ;
        Statement dbstmt2 = null ;

        Connection dbc = parent.dboConnection() ;

        try
        {
            dbstmt1 = dbc.createStatement() ;

            ResultSet rs = dbstmt1.executeQuery("select count(oid) from JOP_Id" ) ;

            rs.next() ;
        }
        catch (SQLException j)
        {
            dbstmt2 = dbc.createStatement() ;

            String s1 = "create table JOP_Id (oid " + parent.typhdl.getDefinition(JavaTypes.TYPE_OBJECT) + ", classname " + parent.typhdl.getDefinition(JavaTypes.TYPE_STRING) + ") " ;

            // System.out.println("SQL: " + s1 ) ;

            dbstmt2.executeUpdate(s1) ;

            // System.out.println("SQL: create unique index JOP_IdOID on JOP_Id (oid)" ) ;

            dbstmt2.executeUpdate("create unique index JOP_IdOID on JOP_Id (oid)") ;

            dbstmt2.close() ;

            dbc.commit() ;
        }

        if (dbstmt1 != null)
        {
            dbstmt1.close() ;
        }
    }

   /**
        This method update the current object's id and
        classname into the special 'JOP_Id'. This table
        is used when an object is retrieved to determine the
        type of the class, and hence which other tables
        should be read.
    */
    void updateId( long xid, String classname ) throws Exception
    {
        Connection dbcon = null ;
        PreparedStatement dbstmt = null ;

        try
        {
            dbcon = parent.allocConnection(parent.CON_ID) ;
            dbstmt = dbcon.prepareStatement("insert into JOP_Id (classname, oid) values (?,?)") ;

            dbstmt.setString( 1, classname ) ;
            dbstmt.setString( 2, new JOP.persist.ObjectIdentifier(xid).toString()) ;

            // System.out.println("SQL: insert into JOP_Id (classname, oid) values (" + classname + "," + new JOP.persist.ObjectIdentifier(xid).toString() + ")") ;

            dbstmt.executeUpdate() ;

            dbstmt.close() ;
        }
        catch (Exception ex)
        {
            if (dbstmt != null)
            {
                dbstmt.close() ;
            }

            throw ex ;
        }
    }

    String readClassname( long xid ) throws Exception
    {
        Connection dbcon = null ;
        Statement dbstmt = null ;

        try
        {
            dbcon = parent.allocConnection(parent.CON_ID) ;

            dbstmt = dbcon.createStatement() ;

            String s1 = "SELECT classname FROM JOP_Id WHERE (oid='" + new JOP.persist.ObjectIdentifier(xid).toString() + "');" ;

            // System.out.println("SQL: " + s1 ) ;

            ResultSet r = dbstmt.executeQuery(s1);

            if (r.next())
            {
                // idCache.put( id, classnameDB.toString().trim()) ;

                String name = r.getString(1).trim() ;

                dbstmt.close() ;

                return name ;
            }
            else
            {
                throw new JOP.persist.ObjectNotFoundException(xid) ;
            }
        }
        catch (Exception ex)
        {
            if (dbstmt != null)
            {
                dbstmt.close() ;
            }

            throw ex ;
        }
    }
}

