/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist.jdbc ;

import JOP.persist.InputController ;
import JOP.persist.ObjectDefinition ;
import JOP.persist.ClassDefinition ;
import JOP.persist.SlotDefinition ;
import JOP.persist.ObjectIdentifier ;

import  java.util.Date ;
import  java.util.Hashtable ;
import  java.util.Enumeration ;
import  java.util.Vector ;
import  java.util.StringTokenizer ;

import  JOP.persist.JavaTypes;

import java.net.URL;
import java.sql.*;

/**
    This class contains methods to read objects from the JDBC data source.
    The class is created by the JdbcPersistManager and called from
    JOP.persist.PersistManager
    <p>
    *@see   JOP.persist.PersistManager
*/
class JdbcInputController implements InputController
{
    JdbcPersistManager  parent ;

    JdbcInputController( JdbcPersistManager p )
    {
        parent = p ;
    }

    /**
        Access the index and returns the name of this class.
    */
    public String readClassname( ObjectIdentifier oid ) throws Exception
    {
        return parent.getIdTable().readClassname( oid.getOid() ) ;
    }

    /**
        Read the object
    */
    public void readObject( ObjectIdentifier oid, ObjectDefinition defObject ) throws Exception
    {
        StringBuffer selectedFields = null ;
        StringBuffer selectedTables = null ;
        StringBuffer whereTables = null ;
        Vector       bindVariables = new Vector() ;

        /*
            First build the single composite SELECT statement
        */
        int column = 1 ;

        for (Enumeration eClass = defObject.classes() ; eClass.hasMoreElements() ; )
        {
            ClassDefinition defClass = (ClassDefinition)eClass.nextElement() ;

            String  tableName = parent.config.tableName(defClass.getClassName()) ;

            if (selectedTables == null)
            {
                selectedTables = new StringBuffer(" FROM ") ;
            }
            else
            {
                selectedTables.append(",") ;
            }

            selectedTables.append(tableName) ;

            if (whereTables == null)
            {
                whereTables = new StringBuffer(" WHERE ") ;
            }
            else
            {
                whereTables.append(" AND ") ;
            }

            whereTables.append(tableName + ".oid = '" + oid.toString() + "'") ;

            for (Enumeration eSlot = defClass.slots() ; eSlot.hasMoreElements() ; )
            {
                SlotDefinition defSlot = (SlotDefinition)eSlot.nextElement() ;

                if (selectedFields == null)
                {
                    selectedFields = new StringBuffer("SELECT ") ;
                }
                else
                {
                    selectedFields.append(",") ;
                }

                selectedFields.append(tableName + "." + parent.config.columnName(defSlot.getName())) ;
            }
        }

        //  finish off the statement

        selectedFields.append(selectedTables) ;
        selectedFields.append(whereTables) ;
        selectedFields.append(";") ;

        // execute select statement

        Connection dbcon = parent.allocConnection(parent.CON_NORMAL) ;
        Statement dbstmt = dbcon.createStatement() ;

        try
        {
            // System.out.println("SQL: " + selectedFields) ;

            ResultSet r = dbstmt.executeQuery(selectedFields.toString()) ;

            //  now we need to set the slot value for each slot
            //  by accessing the bind variable for the slot
            //  and storing it in the slotDefinition structure.

            if (!r.next())
            {
                throw new JdbcPersistException("Missing object contents") ;
            }

            int slot = 1 ;

            for (Enumeration eClass = defObject.classes() ; eClass.hasMoreElements() ; )
            {
                ClassDefinition defClass = (ClassDefinition)eClass.nextElement() ;

                for (Enumeration eSlot = defClass.slots() ; eSlot.hasMoreElements() ; )
                {
                    SlotDefinition defSlot = (SlotDefinition)eSlot.nextElement() ;

                    if (defSlot.isArray())
                    {
                        /*
                            Get the object from the result set
                        */

                        Long arrayid = (Long)parent.typhdl.readType( JavaTypes.TYPE_LONG, r, slot++ ) ;

                        if (arrayid == null)
                        {
                            defSlot.setNullArray() ;
                        }
                        else
                        {
                            readArray(arrayid.longValue(), defClass, defSlot ) ;
                        }
                    }
                    else
                    {
                        int signature = defSlot.getSignatureType() ;

                        if (signature == JavaTypes.TYPE_OBJECT)
                        {
                            defSlot.setRefOid((ObjectIdentifier)parent.typhdl.readType( signature, r, slot++)) ;
                        }
                        else
                        {
                            defSlot.setValue(parent.typhdl.readType( signature, r, slot++)) ;
                        }
                    }
                }
            }

            if (r.next())
            {
                throw new JdbcPersistException("Duplicate object contents") ;
            }

            dbstmt.close() ;
        }
        finally
        {
            if (dbstmt != null)
            {
                dbstmt.close() ;
            }
        }
    }

    private void readArray( long arrayid, ClassDefinition defClass, SlotDefinition defSlot ) throws Exception
    {
        int     signature = defSlot.getSignatureType() ;
        String  arrayName = parent.config.arrayName( defClass.getClassName(), defSlot.getName()) ;
        String  columnName = parent.config.columnName( defSlot.getName()) ;

        StringBuffer s1 = new StringBuffer("SELECT " + arrayName + ".arrayindex" ) ;

        s1.append(", " + arrayName + "." + columnName ) ;

        s1.append(" FROM " + arrayName + " WHERE (" + arrayName + ".arrayid='" + new Long(arrayid).toString() + "')" ) ;

        // System.out.println("SQL: " + s1 ) ;

        Connection dbcon = null ;
        Statement dbstmt = null ;

        try
        {
            dbcon = parent.allocConnection(parent.CON_ARRAY) ;
            dbstmt = dbcon.createStatement() ;

            ResultSet r = dbstmt.executeQuery(s1.toString()) ;

            while (r.next())
            {
                Integer arrayindex = (Integer)parent.typhdl.readType( JavaTypes.TYPE_INTEGER, r, 1 ) ;

                Object obj = parent.typhdl.readType( signature, r, 2 ) ;

                if (signature == JavaTypes.TYPE_OBJECT)
                {
                    defSlot.setRefOid(arrayindex.intValue(), (ObjectIdentifier)obj ) ;
                }
                else
                {
                    defSlot.setValue(arrayindex.intValue(), obj ) ;
                }
            }

            dbstmt.close() ;
        }
        finally
        {
            if (dbstmt != null)
            {
                dbstmt.close() ;
            }
        }
    }
}


