/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist.jdbc ;

import  java.sql.PreparedStatement ;
import  java.sql.ResultSet ;
import  java.sql.SQLException ;

/**
    Interface for each type handler
    </p><p>
    A type handler is a class that is used to manager
    the persistence of a specific Java data type
    to a JDBC database. This interface must be
    implemented by all type handlers.
    </p><p>
    Type handlers are loaded dynamically by JOP according
    to the selected strategy. Type handlers must be in
    the package '<b>JOP.persist.jdbc</b>' and they must have a
    name of the following format:
    <pre>
        TypeHandler_<i>typename</i>_<i>strategy</i>

        where

            TypeHandler is the string "TypeHandler"

            <i>typename</i>    is the short name of a built in Java type
                        (see JOP.perist.JavaTypes)

            <i>strategy</i>    is a number. Numbers must be assigned
                        sequentially starting at 1. There must
                        be no leading zeros.
    </pre>
    TypeHandlers must be public classes and must have
    a public constructor that takes no arguments.
*/
public interface TypeHandler
{
    /**
        Save the 'object' by binding it to column of the Jdbc
        statement 'stmt'. The type is one of the constants
        defined in JOP.persist.JavaTypes.
        <p>
        *@see   JOP.persist.JavaTypes
    */
    public void saveType( PreparedStatement stmt, int column, Object object ) throws SQLException ;

    /**
        Read the column from the result set and return it as an object
        of the correct type (implied by this TypeHandlers name).
    */
    public Object readType( ResultSet rs, int column ) throws SQLException ;

    /**
        Return the name of the SQL data type for this Java type.
    */
    public String getDefinition() ;
}

