/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist.jdbc ;

import  java.sql.* ;

public class TypeHandler_Date_2 implements TypeHandler
{
    public TypeHandler_Date_2()
    {
    }

    /**
        Save the 'object' by binding it to column of the Jdbc
        statement 'stmt'. The type is one of the constants
        defined in JOP.persist.JavaTypes.
    */
    public void saveType( PreparedStatement stmt, int column, Object object ) throws SQLException
    {
        if (object == null)
        {
            stmt.setNull(column, Types.VARCHAR) ;
        }
        else
        {
            stmt.setString( column, dateToString((java.util.Date)object)) ;
        }
    }

    /**
        Read the type and return an object
    */
    public Object readType( ResultSet rs, int column ) throws SQLException
    {
        String s = rs.getString(column) ;

        if (rs.wasNull())
        {
            return null ;
        }
        else
        {
            return dateFromString(s) ;
        }
    }

    public String getDefinition()
    {
        return "char(20)" ;
    }

    public String toString()
    {
        return "Date -> " + getDefinition() ;
    }

    private String dateToString(java.util.Date d)
    {
        String year = new Integer(d.getYear() + 1900).toString().substring(0,4) ;
        String month = new Integer(d.getMonth() + 101).toString().substring(1,3) ;
        String date = new Integer(d.getDate() + 100).toString().substring(1,3) ;
        String hour = new Integer(d.getHours() + 100).toString().substring(1,3) ;
        String minutes = new Integer(d.getMinutes() + 100).toString().substring(1,3) ;
        String seconds = new Integer(d.getSeconds() + 100).toString().substring(1,3) ;

        return year + "-" + month + "-" + date + " " + hour + ":" + minutes + ":" + seconds ;
    }

    private java.util.Date dateFromString(String s)
    {
        return new java.util.Date(
                new Integer(s.substring( 0, 4 )).intValue()-1900,
                new Integer(s.substring( 5, 7 )).intValue()-1,
                new Integer(s.substring( 8, 10 )).intValue(),
                new Integer(s.substring( 11, 13 )).intValue(),
                new Integer(s.substring( 14, 16 )).intValue(),
                new Integer(s.substring( 17, 19 )).intValue() ) ;
    }
}

