/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persist.jdbc ;

import  java.sql.* ;

public class TypeHandler_String_3 implements TypeHandler
{
    public TypeHandler_String_3()
    {
    }

    /**
        Save the 'object' by binding it to column of the Jdbc
        statement 'stmt'. The type is one of the constants
        defined in JOP.persist.JavaTypes.
    */
    public void saveType( PreparedStatement stmt, int column, Object object ) throws SQLException
    {
        if (object == null)
        {
            stmt.setNull(column, Types.VARCHAR) ;
        }
        else
        {
            String out = "'" + object + "'" ;

            char oldc[] = out.toCharArray() ;
            char newc[] = new char[oldc.length * 2] ;
            int len = 0 ;

            for (int i = 0 ; i < oldc.length ; i++)
            {
                if (oldc[i] == '\\')
                {
                    newc[len++] = '\\' ;
                    newc[len++] = '\\' ;
                }
                else if (oldc[i] == '\0')
                {
                    newc[len++] = '\\' ;
                    newc[len++] = '0' ;
                }
                else
                {
                    newc[len++] = oldc[i] ;
                }
            }

            out = new String(newc, 0, len) ;

            if (out.length() > 255)
            {
                throw new SQLException("VARCHAR field too long: " + out.length()) ;
            }

            stmt.setString( column, out) ;
        }
    }

    /**
        Read the type and return an object
    */
    public Object readType( ResultSet rs, int column ) throws SQLException
    {
        String s = rs.getString(column) ;

        if (rs.wasNull())
        {
            return null ;
        }
        else
        {
            char inpc[] = s.toCharArray() ;
            char outc[] = new char[inpc.length] ;
            int len = 0 ;

            for (int i = 0 ; i < inpc.length ; i++)
            {
                if (inpc[i] == '\\')
                {
                    i++ ;

                    if (i >= inpc.length)
                    {
                        throw new RuntimeException("Corrupt escape sequence") ;
                    }

                    if (inpc[i] == '\\')
                    {
                        outc[len++] = '\\' ;
                    }
                    else if (inpc[i] == '0')
                    {
                        outc[len++] = '\0' ;
                    }
                }
                else
                {
                    outc[len++] = (char)((int)inpc[i] & 0xFF) ;
                }
            }

            return new String(outc, 1, len-2 ) ;
        }
    }

    public String getDefinition()
    {
        return "varchar(255)" ;
    }

    public String toString()
    {
        return "String -> " + getDefinition() + " quoted string with escaping" ;
    }
}

