/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persistmarshall.java.lang ;

import  JOP.persist.PersistController ;
import  JOP.persist.PersistException ;
import  JOP.persist.Persistable ;
import  JOP.persist.PersistableArray ;
import  JOP.persist.PersistMarshall ;

/**
    This is the PeristMarshall class that allows the
    standard java.lang.Long to be saved and read
    by a PersistManager.
    <p>
    This class is used when a slot is declared as
    type 'Object' but contains an object of type 'Long'.
    The PersistManager will define the slot as an
    Object and assign a reference, this class will be
    invoked when the Long is marshalled. Without this
    class everything will still work, however the Native
    marshaller will kick-in and the Long will be marshalled
    as an array of characters. An array of characters is
    not a very efficient solution.
    <br>
    *@see   JOP.persist.PersistMarshall
*/

public class Long_JOP implements PersistMarshall
{
    Long  target ;

    public Long_JOP()
    {
    }

    public boolean canModify()
    {
        return false ;
    }

    public void setObject( Object obj ) throws Exception
    {
        if (obj instanceof Long)
        {
            target = (Long)obj ;
        }
        else
        {
            throw new PersistException("Can only marshall java.lang.Long") ;
        }
    }

    public Object getObject()
    {
        if (target == null)
        {
            target = new Long(0) ;
        }

        return target;
    }

    public void defineObject(PersistController c) throws Exception
    {
        c.defineSlot( "JOP.persistmarshall.java.lang.Long_JOP", "value", PersistController.TYPE_LONG ) ;
    }

    public void streamIn(PersistController c) throws Exception
    {
        target = c.getLong("value") ;
    }

    public void streamOut(PersistController c) throws Exception
    {
        c.put( "value", target ) ;
    }
}

