/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persistmarshall.java.util ;

import  JOP.persist.PersistController ;
import  JOP.persist.PersistException ;
import  JOP.persist.Persistable ;
import  JOP.persist.PersistableArray ;
import  JOP.persist.PersistMarshall ;

import  java.util.Date ;

/**
    This is the PeristMarshall class that allows the
    standard java.util.Date to be saved and read
    by a PersistManager.
    <p>
    This class is used when a slot is declared as
    type 'Object' but contains an object of type 'Date'.
    The PersistManager will define the slot as an
    Object and assign a reference, this class will be
    invoked when the Date is marshalled. Without this
    class everything will still work, however the Native
    marshaller will kick-in and the Date will be marshalled
    as its internal structure, and not as a 'date'.
    <p>
    *@see   JOP.persist.PersistMarshall
*/

public class Date_JOP implements PersistMarshall
{
    Date  target ;

    public Date_JOP()
    {
    }

    public boolean canModify()
    {
        return false ;
    }

    public void setObject( Object obj ) throws Exception
    {
        if (obj instanceof Date)
        {
            target = (Date)obj ;
        }
        else
        {
            throw new PersistException("Can only marshall java.lang.Date") ;
        }
    }

    public Object getObject()
    {
        if (target == null)
        {
            target = new Date() ;
        }

        return target;
    }

    public void defineObject(PersistController c) throws Exception
    {
        c.defineSlot( "JOP.persistmarshall.java.lang.Date_JOP", "value", PersistController.TYPE_DATE ) ;
    }

    public void streamIn(PersistController c) throws Exception
    {
        target = c.getDate("value") ;
    }

    public void streamOut(PersistController c) throws Exception
    {
        c.put( "value", target ) ;
    }
}


