/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persistmarshall.java.util ;

import  JOP.persist.PersistController ;
import  JOP.persist.PersistException ;
import  JOP.persist.Persistable ;

import  JOP.persist.Hashcode ;

import  java.util.Date ;

public class HashtableElement implements Persistable
{
    Object      value ;
    Object      key ;
    private transient boolean init = false ;

    private static final int ELEMENT_CHARACTER = 1 ;
    private static final int ELEMENT_INTEGER = 2 ;
    private static final int ELEMENT_LONG = 3 ;
    private static final int ELEMENT_FLOAT = 4 ;
    private static final int ELEMENT_DOUBLE = 5 ;
    private static final int ELEMENT_STRING = 6 ;
    private static final int ELEMENT_DATE = 7 ;
    private static final int ELEMENT_OBJECT = 8 ;
    private static final int ELEMENT_REF = 9 ;

    int calcHashcode()
    {
        Hashcode hash = new Hashcode() ;

        hash.calc(value) ;
        hash.calc(key) ;

        return hash.getHashcode() ;
    }

    public String toString()
    {
        return key + " -> " + value ;
    }

    public HashtableElement()
    {
    }

    HashtableElement( Object k, Object obj )
    {
        init = true ;

        value = obj ;
        key = k ;
    }

    public void defineObject( PersistController c ) throws Exception
    {
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "keyType", PersistController.TYPE_INTEGER ) ;

        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "keyCharacter", PersistController.TYPE_CHARACTER ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "keyInteger", PersistController.TYPE_INTEGER ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "keyLong", PersistController.TYPE_LONG ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "keyFloat", PersistController.TYPE_FLOAT ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "keyDouble", PersistController.TYPE_DOUBLE ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "keyString", PersistController.TYPE_STRING ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "keyDate", PersistController.TYPE_DATE ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "keyObject", PersistController.TYPE_OBJECT ) ;

        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refType", PersistController.TYPE_INTEGER ) ;

        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refCharacter", PersistController.TYPE_CHARACTER ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refInteger", PersistController.TYPE_INTEGER ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refLong", PersistController.TYPE_LONG ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refFloat", PersistController.TYPE_FLOAT ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refDouble", PersistController.TYPE_DOUBLE ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refString", PersistController.TYPE_STRING ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refDate", PersistController.TYPE_DATE ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refObject", PersistController.TYPE_OBJECT ) ;
        c.defineSlot( "JOP.persistmarshall.java.util.HashtableElement", "refRef", PersistController.TYPE_LONG ) ;
    }

    public void streamIn( PersistController c ) throws Exception
    {
        int keyType = c.intValue("keyType") ;

        switch (keyType)
        {
            case ELEMENT_CHARACTER :
                key = c.getCharacter("keyCharacter") ;
                break ;
            case ELEMENT_INTEGER :
                key = c.getInteger("keyInteger") ;
                break ;
            case ELEMENT_LONG :
                key = c.getLong("keyLong") ;
                break ;
            case ELEMENT_FLOAT :
                key = c.getFloat("keyFloat") ;
                break ;
            case ELEMENT_DOUBLE :
                key = c.getDouble("keyDouble") ;
                break ;
            case ELEMENT_STRING :
                key = c.getString("keyString") ;
                break ;
            case ELEMENT_DATE :
                key = c.getDate("keyDate") ;
                break ;
            case ELEMENT_OBJECT :
                key = c.getObject("keyObject") ;
                break ;
            default :
                throw new PersistException("Unknown hashtable key type '" + keyType + "'") ;
        }

        int refType = c.intValue("refType") ;

        switch (refType)
        {
            case ELEMENT_CHARACTER :
                value = c.getCharacter("refCharacter") ;
                break ;
            case ELEMENT_INTEGER :
                value = c.getInteger("refInteger") ;
                break ;
            case ELEMENT_LONG :
                value = c.getLong("refLong") ;
                break ;
            case ELEMENT_FLOAT :
                value = c.getFloat("refFloat") ;
                break ;
            case ELEMENT_DOUBLE :
                value = c.getDouble("refDouble") ;
                break ;
            case ELEMENT_STRING :
                value = c.getString("refString") ;
                break ;
            case ELEMENT_DATE :
                value = c.getDate("refDate") ;
                break ;
            case ELEMENT_REF :
                value = new JOP.persist.ObjectIdentifier(c.longValue("refRef")) ;
                break ;
            case ELEMENT_OBJECT :
                value = c.getObject("refObject") ;
                break ;
            default :
                throw new PersistException("Unknown hashtable object type '" + refType + "'") ;
        }

        init = true ;
    }

    public void streamOut( PersistController c ) throws Exception
    {
        if (key instanceof Character)
        {
            Character v = (Character)key ;

            c.put("keyType", ELEMENT_CHARACTER ) ;
            c.put("keyCharacter", v ) ;
        }
        else if (key instanceof Integer)
        {
            Integer v = (Integer)key ;

            c.put("keyType", ELEMENT_INTEGER ) ;
            c.put("keyInteger", v ) ;
        }
        else if (key instanceof Long)
        {
            Long v = (Long)key ;

            c.put("keyType", ELEMENT_LONG ) ;
            c.put("keyLong", v ) ;
        }
        else if (key instanceof Float)
        {
            Float v = (Float)key ;

            c.put("keyType", ELEMENT_FLOAT ) ;
            c.put("keyFloat", v ) ;
        }
        else if (key instanceof Double)
        {
            Double v = (Double)key ;

            c.put("keyType", ELEMENT_DOUBLE ) ;
            c.put("keyDouble", v ) ;
        }
        else if (key instanceof String)
        {
            String v = (String)key ;

            c.put("keyType", ELEMENT_STRING) ;
            c.put("keyString", v ) ;
        }
        else if (key instanceof Date)
        {
            Date v = (Date)key ;

            c.put("keyType", ELEMENT_DATE ) ;
            c.put("keyDate", v ) ;
        }
        else
        {
            c.put("keyType", ELEMENT_OBJECT ) ;
            c.put("keyObject", key ) ;
        }

        /***** Now the reference value *****/

        if (value instanceof Character)
        {
            Character v = (Character)value ;

            c.put("refType", ELEMENT_CHARACTER ) ;
            c.put("refCharacter", v ) ;
        }
        else if (value instanceof Integer)
        {
            Integer v = (Integer)value ;

            c.put("refType", ELEMENT_INTEGER ) ;
            c.put("refInteger", v ) ;
        }
        else if (value instanceof Long)
        {
            Long v = (Long)value ;

            c.put("refType", ELEMENT_LONG ) ;
            c.put("refLong", v ) ;
        }
        else if (value instanceof Float)
        {
            Float v = (Float)value ;

            c.put("refType", ELEMENT_FLOAT ) ;
            c.put("refFloat", v ) ;
        }
        else if (value instanceof Double)
        {
            Double v = (Double)value ;

            c.put("refType", ELEMENT_DOUBLE ) ;
            c.put("refDouble", v ) ;
        }
        else if (value instanceof String)
        {
            String v = (String)value ;

            c.put("refType", ELEMENT_STRING) ;
            c.put("refString", v ) ;
        }
        else if (value instanceof Date)
        {
            Date v = (Date)value ;

            c.put("refType", ELEMENT_DATE ) ;
            c.put("refDate", v ) ;
        }
        else if (value instanceof JOP.persist.ObjectIdentifier)
        {
            Long v = new Long(((JOP.persist.ObjectIdentifier)value).getOid()) ;

            c.put("refType", ELEMENT_REF ) ;
            c.put("refRef", v ) ;
        }
        else
        {
            c.put("refType", ELEMENT_OBJECT ) ;
            c.put("refObject", value ) ;
        }
    }
}

