/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persistmarshall.java.util ;

import  JOP.persist.PersistController ;
import  JOP.persist.PersistException ;
import  JOP.persist.Persistable ;
import  JOP.persist.PersistableArray ;
import  JOP.persist.PersistMarshall ;
import  JOP.persist.Hashcode ;

import  java.util.Date ;
import  java.util.Hashtable ;
import  java.util.Enumeration ;

/**
    This is the PeristMarshall class that allows the
    standard java.util.Hashtable to be saved and read
    by a PersistManager.
    <p>
    *@see   JOP.persist.PersistMarshall
*/

public class Hashtable_JOP implements PersistMarshall
{
    private Hashtable   target ;

    /*
        Because we create a new HashtableElement every time
        we stream a hashtable in or out the JOP optimisation
        that detects whether an object has changed does not
        work. In effect, Hashtables always appear to change.
        To counteract this effect we store a 'hashed'
        version of all the element objects that we
        have used and re-use it if possible.
    */

    private static Hashtable   elementCache = new Hashtable() ;

    public Hashtable_JOP()
    {
    }

    public boolean canModify()
    {
        return true ;
    }

    public void setObject( Object obj ) throws Exception
    {
        if (obj instanceof Hashtable)
        {
            target = (Hashtable)obj ;
        }
        else
        {
            throw new PersistException("Can only marshall java.util.Hashtable") ;
        }
    }

    public Object getObject()
    {
        if (target == null)
        {
            target = new Hashtable() ;
        }

        return target;
    }

    public void defineObject(PersistController c) throws Exception
    {
        c.defineSlotArray( "JOP.persistmarshall.java.util.Hashtable_JOP", "elements", PersistController.TYPE_OBJECT ) ;
    }

    /**
        Load the hashtable from the persist manager. The
        hashtable is stored as a number of HashtableElements.
    */
    public void streamIn(PersistController c) throws Exception
    {
        Object temp[] = c.getObjectArray("elements") ;

        if (target == null)
        {
            target = new Hashtable() ;
        }

        // System.out.println("Vector elements = " + temp.length ) ;

        if (temp != null)
        {
            for (int i = 0 ; i < temp.length ; i++)
            {
                HashtableElement element = (HashtableElement)temp[i] ;

                // System.out.println("IN Element[" + i + "] = " + element.key + " -> " + element.value ) ;

                target.put( element.key, element.value ) ;

                // System.out.println("SAVED ELEMENT " + element.calcHashcode() + "(" + element.key + " -> " + element.value ) ;

                elementCache.put(new Integer(element.calcHashcode()), element) ;

                // element.value = null ;
                // element.key = null ;
            }
        }

        // System.out.println("Hashtable = " + target ) ;
    }

    public void streamOut(PersistController c) throws Exception
    {
        Object elements[] = new HashtableElement[target.size()] ;

        int i = 0 ;

        // System.out.println("OUT: Vector elements = " + elements.length ) ;

        for (Enumeration e = target.keys() ; e.hasMoreElements() ; )
        {
            Object  key = e.nextElement() ;
            Object  obj = target.get(key) ;

            elements[i] = getCachedHashtableElement( key, obj ) ;

            // System.out.println("OUT Element[" + i + "] = " + elements[i].key + " -> " + elements[i].value ) ;

            i++ ;
        }

        c.put( "elements", elements ) ;
    }

    private HashtableElement getCachedHashtableElement( Object key, Object obj )
    {
        Hashcode hash = new Hashcode() ;

        hash.calc(obj) ;
        hash.calc(key) ;

        Integer val = new Integer(hash.getHashcode()) ;

        // System.out.println("TRY TO GET ELEMENT " + val + "(" + key + " -> " + obj ) ;

        HashtableElement h = (HashtableElement)elementCache.get(val) ;

        if (h == null)
        {
            h = new HashtableElement(key, obj) ;

            // System.out.println("SAVED NEW ELEMENT " + h.calcHashcode()) ;

            elementCache.put(new Integer(h.calcHashcode()), h) ;
        }
        else
        {
            h.key = key ;
            h.value = obj ;
        }

        return h ;
    }
}

