/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.persistmarshall.java.util ;

import  JOP.persist.PersistController ;
import  JOP.persist.PersistException ;
import  JOP.persist.Persistable ;
import  JOP.persist.PersistableArray ;
import  JOP.persist.PersistMarshall ;

import  java.util.Date ;
import  java.util.Vector ;
import  java.util.Enumeration ;

/**
    This is the PeristMarshall class that allows the
    standard java.util.Vector to be saved and read
    by a PersistManager.
    <p>
    *@see   JOP.persist.PersistMarshall
*/

public class Vector_JOP implements PersistMarshall
{
    Vector  target ;

    public Vector_JOP()
    {
    }

    /**
        Vectors can be modified once created so returns 'true'
    */
    public boolean canModify()
    {
        return true ;
    }

    public void setObject( Object obj ) throws Exception
    {
        if (obj instanceof Vector)
        {
            target = (Vector)obj ;
        }
        else
        {
            throw new PersistException("Can only marshall vectors") ;
        }
    }

    public Object getObject()
    {
        if (target == null)
        {
            target = new Vector() ;
        }

        return target;
    }

    public void defineObject(PersistController c) throws Exception
    {
        c.defineSlotArray( "JOP.persistmarshall.java.util.Vector_JOP", "elements", PersistController.TYPE_OBJECT ) ;
    }

    public void streamIn(PersistController c) throws Exception
    {
        Object temp[] = c.getObjectArray("elements") ;

        if (target == null)
        {
            target = new Vector() ;
        }

        // System.out.println("Vector elements = " + temp.length ) ;

        if (temp != null)
        {
            for (int i = 0 ; i < temp.length ; i++)
            {
                target.addElement(temp[i]) ;
            }
        }

        // System.out.println("Vector = " + target ) ;
    }

    public void streamOut(PersistController c) throws Exception
    {
        Object elements[] = new Object[target.size()] ;

        int i = 0 ;

        for (Enumeration e = target.elements() ; e.hasMoreElements() ; )
        {
            Object obj = e.nextElement() ;

            // System.out.println("Add [" + i + "] = '" + obj + "'") ;

            elements[i] = obj ;

            i++ ;
        }

        c.put( "elements", elements ) ;
    }
}


