/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.test ;

import JOP.persist.file.FilePersistManager ;
import JOP.test.PersistTest ;

/**
    This is the 'mainline' for the test program. The purpose
    of this class is to accept command line arguments,
    open a FilePersistManager and then to instantiate and
    invoke the chosen test class.
    <p>
    This class is responsible for inialising and
    closing the FilePersistManager.
    <p>
    The '<i>test_class</i>' is responsible for starting a unit of
    work, saving and retrieving objects and commiting or aborting
    the unit of work.
*/

public class FileTest
{
    /**
        There command line usage is:
        <pre>
            java FileTest <i>test_class</i> filename
        </pre>
    */

    public static void main( String args[] )
    {
        try
        {
            FilePersistManager pm ;

            if (args.length == 2)
            {
                System.out.println(" 0 = " + args[0] ) ;
                System.out.println(" 1 = " + args[1] ) ;

                pm = new FilePersistManager( args[1] ) ;
            }
            else
            {
                System.err.println() ;
                System.err.println("Usage: java FileTest test filename") ;
                System.err.println() ;
                System.err.println("      test          The name of a test class") ;
                System.err.println("      filename      Filename to store object") ;

                return ;
            }

            PersistTest test ;

            try
            {
                test = (PersistTest)(Class.forName(args[0])).newInstance() ;
            }
            catch (Exception e)
            {
                try {
                    test = (PersistTest)(Class.forName("JOP.test." + args[0])).newInstance() ;
                } catch (Exception ex)
                {

                    System.err.println("Test '" + args[0] + "' not found") ;
                    return ;
                }
            }

            pm.open() ;

            test.runTest( pm ) ;

            pm.close() ;
        }
        catch (Exception ex)
        {
            ex.printStackTrace() ;
        }
    }
}

