/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.test ;

import JOP.test.PersistTest ;

import JOP.persist.jdbc.JdbcPersistManager ;

/**
    This is the 'mainline' for the test program. The purpose
    of this class is to accept command line arguments,
    open a JdbcPersistManager and then to instantiate and
    invoke the chosen test class.
    <p>
    This class is responsible for inialising and
    closing the JdbcPersistManager.
    <p>
    The '<i>test_class</i>' is responsible for starting a unit of
    work, saving and retrieving objects and commiting or aborting
    the unit of work.
*/

public class JdbcTest
{
    /**
        There are two possible command line usages.
        <pre>

            java JdbcTest test_class driver url username password

        or

            java JdbcTest test_class driver url username password dbousername dbopassword

        </pre>
        The first approach uses a single DB account to
        access objects and also create and drop tables (when
        necessary). If the acocunt does not have sufficient
        priveledge to create/drop tables then an error may
        occur if the PersistManager needs to create or
        drop tables.
        <p>
        The second example allows two accounts to be specified,
        one for normal access and the second to create/drop
        tables.
        <p>
        *@see   JOP.persist.jdbc.JdbcPersistManager
    */

    public static void main( String args[] )
    {
        /*
                opens the Jdbc controller
        */
        try
        {
            JdbcPersistManager pm ;

            System.err.println("Args = " + args.length) ;

            if (args.length == 5)
            {
                pm = new JdbcPersistManager( args[1], args[2], args[3], args[4] ) ;
            }
            else
            {
                System.err.println() ;
                System.err.println("Usage: java XDBTest test driver url username password [ dbousername dbopassword ]") ;
                System.err.println() ;
                System.err.println("      test          The name of a test class (eg Test1)") ;
                System.err.println("      driver        Driver name (eg jdbc.odbc.JdbcOdbcDriver)") ;
                System.err.println("      url           URL for data source (eg jdbc:odbc:test)") ;
                System.err.println("      username      User name for default DB access") ;
                System.err.println("      password      Password for default access") ;

                return ;
            }

            PersistTest test ;

            try
            {
                test = (PersistTest)(Class.forName(args[0])).newInstance() ;
            }
            catch (Exception e)
            {
                try {
                    test = (PersistTest)(Class.forName("JOP.test." + args[0])).newInstance() ;
                } catch (Exception ex)
                {

                    System.err.println("Test '" + args[0] + "' not found") ;
                    return ;
                }
            }


            pm.open() ;

            test.runTest( pm ) ;

            pm.close() ;

        } catch (Exception ex)
        {
            ex.printStackTrace() ;
        }
    }
}

