/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.test ;

import JOP.test.PersistTest ;
import  JOP.test.TestObject0 ;

import  JOP.persist.PersistManager ;
import  JOP.persist.ObjectIdentifier ;

/**
    This test validates object modification and unit of work
    support.
*/

public class Test10 implements PersistTest
{
    /**
        Entry point called by the test harness
    */
    public void runTest( PersistManager pm )
    {
        System.out.println("\n**** Test object modification and units of work ****\n");

        try {
            // first create an object

            pm.beginWork() ;

            TestObject0 t1 = new TestObject0() ;

            t1.setValues() ;

            System.out.println("Saving object...") ;

            long id = pm.streamOutObject( t1 ) ;

            System.out.println("Object saved as id = " + new ObjectIdentifier(id) ) ;

            // now read back the object - within the unit of work

            System.out.println("Read object within unit of work...") ;

            TestObject0 t2 = (TestObject0)pm.streamInObject( id ) ;

            if (t1 != t2)       // objects should be identical
            {
                throw new Exception("Object not read correctly within unit of work") ;
            }

            pm.commitWork() ;

            pm.beginWork() ;

            System.out.println("Read object in new unit of work...") ;

            TestObject0 t3 = (TestObject0)pm.streamInObject( id ) ;

            if (t1 == t3)       // objects should NOT be identical
            {
                throw new Exception("Object not read correctly outside of unit of work") ;
            }

            // now change t3

            t3.z = "Modified" ;

            System.out.println("Save modified object...") ;

            long id2 = pm.streamOutObject( t3 ) ;

            if (id != id2)
            {
                throw new Exception("Object saved with different id") ;
            }

            pm.commitWork() ;

            pm.beginWork() ;

            System.out.println("Read modified object in new unit of work...") ;

            TestObject0 t4 = (TestObject0)pm.streamInObject( id ) ;

            if (t4 == t1 || t4 == t3)       // objects should NOT be identical
            {
                throw new Exception("Object not read correctly after modification") ;
            }

            System.out.println(t4) ;

            if (!t4.z.equals("Modified"))
            {
                throw new Exception("Modification not read back") ;
            }

            System.out.println("Completed OK") ;

            pm.commitWork() ;

        } catch (Exception ex)
        {
            ex.printStackTrace() ;

            try {
                pm.abortWork() ;
            } catch (Exception e)
            {
                System.err.println(e.getMessage()) ;
            }
        }
    }
}




