/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.test ;

import JOP.test.PersistTest ;
import  JOP.test.TestObject0 ;

import  JOP.persist.PersistManager ;
import  JOP.persist.ObjectIdentifier ;
import  JOP.util.HashtableDB ;

/**
    This test creates an object with a user-defined, fixed 'id' and
    verifies to make sure it is read with the same 'id'.
    <p>
    If an object already exists with the defined 'id' then
    that object is read and displayed.
*/

public class Test11 implements PersistTest
{
    private final static long   id = JOP.persist.ObjectIdentifier.USER_LOW+1 ;

    /**
        Entry point called by the test harness
    */
    public void runTest( PersistManager pm )
    {
        System.out.println("\n**** User-defined object identifiers ****\n");

        try {
            // first try and read the object

            pm.beginWork() ;

            HashtableDB h1 = null ;

            try
            {
                h1 = (HashtableDB)pm.streamInObject(id) ;

                System.out.println("Object " + id + " already exists") ;
            }
            catch (JOP.persist.ObjectNotFoundException ex)
            {
                System.out.println("Object does not exist") ;

                // create the object

                h1 = new HashtableDB() ;

                TestObject0 t1 = new TestObject0() ;

                t1.setValues() ;

                h1.put( "TEST1", t1 ) ;

                TestObject3 t3 = new TestObject3() ;

                t3.setValues() ;

                h1.put( "TEST3", t3 ) ;

                System.out.println("Saving object...as id " + id) ;

                JOP.persist.ObjectIdentifier.defineUserOid( h1, id ) ;

                long xid = pm.streamOutObject( h1 ) ;

                System.out.println("Object saved as id = " + new ObjectIdentifier(xid) ) ;
            }

            pm.commitWork() ;

        } catch (Exception ex)
        {
            ex.printStackTrace() ;

            try {
                pm.abortWork() ;
            } catch (Exception e)
            {
                System.err.println(e.getMessage()) ;
            }
        }
    }
}




