/*
    Copyright (c) 1996, David Rothwell. All Rights Reserved.

    http://www.magna.com.au/~davidr     mailto:davidr@magna.com.au

    Permission to use, copy, modify, and distribute this software
    and its documentation for NON-COMMERCIAL purposes and without
    fee is hereby granted provided that this copyright notice
    appears in all copies. This software is distributed on an 'as is'
    basis without warranty.

    Release: 0.4a   11-Sep-1996

 **/

package JOP.test ;

import JOP.test.PersistTest ;

import  JOP.persist.PersistManager ;

import  java.io.DataInputStream ;

import  java.util.Hashtable ;
import  java.util.Enumeration ;
import  JOP.util.HashtableDB ;

/**
    This test allows objects to be retrieved from the persistent
    database using their object identifier and then displays
    the objects.
*/
public class Test2 implements PersistTest
{
    PersistManager  dump ;

    /**
        Entry point called by the test harness
    */
    public void runTest( PersistManager pm )
    {
        System.out.println("\n**** Retrieve saved objects ****\n");

        try {
            pm.beginWork() ;

            boolean done = false ;

            while (!done)
            {
                Long id = null ;

                try
                {
                    DataInputStream d = new DataInputStream(System.in) ;

                    System.out.println() ;
                    System.out.print("ready> ") ;
                    System.out.flush() ;

                    String l = d.readLine() ;

                    if (l.equalsIgnoreCase("q"))
                    {
                        done = true ;
                    }
                    else if (l.equalsIgnoreCase("h") || l.equalsIgnoreCase("?"))
                    {
                        System.out.println() ;
                        System.out.println("q - quit") ;
                        System.out.println("q - commit") ;
                        System.out.println("g - generate an object identifier") ;
                    }
                    else if (l.equalsIgnoreCase("c"))
                    {
                        System.out.println("commit...") ;
                        System.out.flush() ;

                        pm.commitWork() ;
                        pm.beginWork() ;
                    }
                    else if (l.equalsIgnoreCase("g"))
                    {
                        long xid = JOP.persist.OidGenerator.newOid() ;

                        System.out.println(xid) ;
                        String ids = JOP.persist.ObjectIdentifier.oid(xid) ;
                        System.out.println(ids) ;
                        System.out.println(JOP.persist.ObjectIdentifier.oid(ids)) ;
                    }
                    else
                    {
                        if (l.length() > 5 && l.charAt(4) == '-')
                        {
                            id = new Long(JOP.persist.ObjectIdentifier.oid(l)) ;
                        }
                        else
                        {
                            id = new Long(l) ;
                        }

                        /*
                            if we parsed the command input ok
                            then get an object and print it
                            out... We assume it supports
                            toString() in this demo application.
                        */

                        try
                        {
                            Object t = pm.streamInObject(id.longValue()) ;

                            processObject( t ) ;

                        } catch (Exception ex)
                        {
                            System.out.println() ;
                            ex.printStackTrace() ;
                            System.out.println() ;
                        }
                    }
                } catch (Exception ex)
                {
                    // ignore
                }
            }

            pm.commitWork() ;

        } catch (Exception ex)
        {
            ex.printStackTrace() ;

            try {
                pm.abortWork() ;
            } catch (Exception e)
            {
                System.err.println(e.getMessage()) ;
            }
        }
    }

    void processObject( Object t)
    {
        if (t == null)
        {
            System.out.println("************* null *************" ) ;
        }
        else
        {
            System.out.println("************* " + t.getClass().getName() + " *************" ) ;
        }

        if ((t instanceof Hashtable) ||
            (t instanceof HashtableDB))
        {
            hashtableAccess( t ) ;
        }
        else
        {
            System.out.println(t) ;

            System.out.println("*************") ;
        }
    }

    void hashtableAccess( Object t)
    {
        String key ;

        while (true)
        {
            try
            {
                DataInputStream d = new DataInputStream(System.in) ;

                System.out.println() ;
                System.out.print("key> ") ;
                System.out.flush() ;

                key = d.readLine() ;

                if (key.equalsIgnoreCase("h") || key.equalsIgnoreCase("?"))
                {
                    System.out.println() ;
                    System.out.println("=n  - dump element number 'n'") ;
                    System.out.println("d   - display all elements") ;
                    System.out.println("key - display the element with a given key value") ;
                    System.out.println("q   - return to ready>") ;
                }
                if (key.startsWith("="))
                {
                    // get specific element

                    int element = new Integer(key.substring(1)).intValue() + 1 ;
                    Object o = null ;
                    Object k = null ;

                    if (t instanceof Hashtable)
                    {
                        Hashtable ht = (Hashtable) t ;

                        for (Enumeration e = ht.keys() ; element > 0 && e.hasMoreElements() ; )
                        {
                            k = e.nextElement() ;
                            element-- ;
                        }

                        o = ht.get(k) ;

                    }
                    else if (t instanceof HashtableDB)
                    {
                        HashtableDB ht = (HashtableDB) t ;

                        for (Enumeration e = ht.keys() ; element > 0 && e.hasMoreElements() ; )
                        {
                            k = e.nextElement() ;
                            element-- ;
                        }

                        o = ht.get(k) ;
                    }

                    if (element > 0)
                    {
                        System.out.println("No element") ;
                    }
                    else
                    {
                        System.out.println("key = " + k ) ;

                        processObject( o ) ;
                    }
                }
                else if (key.equals("d"))
                {
                    // dump out the contents

                    if (t instanceof Hashtable)
                    {
                        Hashtable ht = (Hashtable) t ;

                        int i = 0 ;

                        for (Enumeration e = ht.keys() ; e.hasMoreElements() ; )
                        {
                            System.out.println("key[" + i + "] = " + e.nextElement()) ;
                            i++ ;
                        }
                    }
                    else if (t instanceof HashtableDB)
                    {
                        HashtableDB ht = (HashtableDB) t ;

                        int i = 0 ;

                        for (Enumeration e = ht.keys() ; e.hasMoreElements() ; )
                        {
                            System.out.println("key[" + i + "] = " + e.nextElement()) ;
                            i++ ;
                        }
                    }

                }
                else if (key.equals("q"))
                {
                    return ;
                }
                else
                {
                    Object o = null ;

                    if (t instanceof Hashtable)
                    {
                        Hashtable ht = (Hashtable) t ;

                        o = ht.get( key ) ;
                    }
                    else if (t instanceof HashtableDB)
                    {
                        HashtableDB ht = (HashtableDB) t ;

                        o = ht.get( key ) ;
                    }

                    processObject( o ) ;
                }

            } catch (Exception ex)
            {
                key = null ;
            }
        }
    }
}

